/*
 * BookMark.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: BookMark.java,v 1.2 2003/04/09 15:54:02 brisk Exp $
 */

import java.io.*;

/**
 * <code>BookMark</code> class is used to read/write PilotDoc bookmark records
 * @author Jeffrey A. Krzysztow
 * @author Reinhold F. Kainhofer
 * @author Pat Beirne
 * @version 1.2
 */
public class BookMark {
	/**
	 * Default constructor, don't do anything
	 */
	BookMark() {}

	/**
	 * Constructor to duplicate an instance of this class
	 */
	BookMark(BookMark bmk) {
		name = bmk.name;
		fileOffset = bmk.fileOffset;
	}

	/**
	 * the name of the bookmark
	 * @since 1.0
	 */
	public String name = ""; 	// 16 Bytes	Bookmark name

	/**
	 * offset into PilotDoc for this bookmark
	 * @since 1.0
	 */
	public int fileOffset = -1;	// 4 DWord	file offset
								// 20 bytes bookmark

	/**
	 * The size of the BookMark in bytes
	 * @return the number of bytes in the BookMark
	 * @since 1.0
	 */
	public static int getSize() {
		return(20);
	}

	/**
	 * Reads the BookMark from the DataInput
	 * @param the DataInput to read from
	 * @since 1.0
	 */
	public void read(DataInput di) throws IOException {
		byte[] b = new byte[16];
		di.readFully(b);
		name = new String(b);
		int i = name.indexOf('\0');
		if(i >= 0) {
			name = name.substring(0, i);
		}
		fileOffset = di.readInt();
	}

	/**
	 * Writes the BookMark to the DataInput
	 * @param the DataInput to write to
	 * @since 1.0
	 */
	public void write(DataOutput out) throws IOException {
		byte[] b = new byte[16];
		byte[] bName = name.getBytes();
		for(int x=0; x < b.length; x++) {
			if(x < bName.length) {
				b[x] = bName[x];
			}
			else {
				b[x] = 0;
			}
		}
		out.write(b);
		out.writeInt(fileOffset);
	}

	/**
	 * override Object.toString()
	 * @since 1.0
	 */
	public String toString() {
		return ">> BookMark <<\nname = `" + name
			+ "`\nfileOffset = " + fileOffset;
	}
}

