/*
 * DecodePanel.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DecodePanel.java,v 1.4 2006/02/08 05:03:06 brisk Exp $
 */

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

// NOTE: for JDK 1.1.x get special collections
// from Sun http://java.sun.com/products/javabeans/infobus/collectionsreadme.html
// import com.sun.java.util.collections.*;
// NOTE: for JDK 1.2+
// import java.util.*;
import @COLLECTIONS@; // used by ant

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * <code>DecodePanel</code> is the Panel with all the decoding options.
 * @author Jeffrey A. Krzysztow
 * @version 1.3.0
 */
public final class DecodePanel extends JPanel implements ProcessDOC {

	public DecodePanel(MakeDocGUI makeDocGUI) {
		super(true);

		this.makeDocGUI = makeDocGUI;

		options = new DecodeOptions();

		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = null;
		JPanel panel = null;

		layout = new GridBagLayout();
		constraints = new GridBagConstraints();
		constraints.insets = new Insets(4, 4, 4, 4);

		setLayout(layout);

		constraints.anchor = GridBagConstraints.WEST;
		constraints.gridwidth = GridBagConstraints.REMAINDER;

		panel = getSourceDestinationPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		panel = getProcessingOptionsPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		@JAVA2_START@
		new DropTarget(sourcePath, new DropTargetListener() {
						   public void drop(DropTargetDropEvent dtde) {
							   try {
								   Transferable tr = dtde.getTransferable();
								   if(dtde.isDataFlavorSupported(DataFlavor.stringFlavor)) {
									   dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
									   String filename = (String)tr.getTransferData(DataFlavor.stringFlavor);
									   sourcePath.setText(filename);
									   dtde.dropComplete(true);
								   }
								   else if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
									   dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
									   java.util.List files = (java.util.List)tr.getTransferData(DataFlavor.javaFileListFlavor);
									   for(Iterator iterator = (Iterator)files.iterator(); iterator.hasNext();) {
										   File file = (File)iterator.next();
										   sourcePath.setText(file.getAbsolutePath());
										   break; // only process first file in the list, for now
									   }
									   dtde.dropComplete(true);
								   }
								   else {
									   dtde.rejectDrop();
								   }
							   }
							   catch(IOException ioe) {
								   ioe.printStackTrace();
							   }
							   catch(UnsupportedFlavorException ufe) {
								   ufe.printStackTrace();
							   }
						   }
						   public void dragExit(DropTargetEvent dte) {
						   }
						   public void dragEnter(DropTargetDragEvent dtde) {
						   }
						   public void dragOver(DropTargetDragEvent dtde) {
						   }
						   public void dropActionChanged(DropTargetDragEvent dtde) {
						   }
					   });
		@JAVA2_END@
	}

	private MakeDocGUI makeDocGUI = null;

	private JPanel getSourceDestinationPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Source: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.weightx = 1;
		sourcePath = new JTextField(30);
		layout.setConstraints(sourcePath, constraints);
		panel.add(sourcePath);
		label.setLabelFor(sourcePath);
		label.setDisplayedMnemonic('s');
		label.setToolTipText("Specifies the PilotDOC containing the text to be converted.");

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0;
		button = new JButton("...");
		button.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										makeDocGUI.browse(sourcePath, "Source file", makeDocGUI.pilotDocFilter);
									}
								});
		layout.setConstraints(button, constraints);
		panel.add(button);

		constraints.gridwidth = 1;
		label = new JLabel("Destination: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.weightx = 1;
		destinationPath = new JTextField(30);
		layout.setConstraints(destinationPath, constraints);
		panel.add(destinationPath);
		label.setLabelFor(destinationPath);
		label.setDisplayedMnemonic('d');
		label.setToolTipText("Specifies the text extracted from the PilotDOC.");

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0;
		button = new JButton("...");
		button.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										makeDocGUI.browse(destinationPath, "Destination file", makeDocGUI.textFilter);
									}
								});
		layout.setConstraints(button, constraints);
		panel.add(button);

		return(panel);
	}

	private JPanel getProcessingOptionsPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JPanel subPanel = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		subPanel = getEOLPanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		bookmarks = new JCheckBox("Create bookmark file:", options.bookmarks);
		bookmarks.addActionListener(new ActionListener() {
									   public void actionPerformed(ActionEvent e) {
										   options.bookmarks = ((JCheckBox)e.getSource()).isSelected();
									   }
								   });
		bookmarks.setMnemonic('c');
		bookmarks.setToolTipText("Used to indicate if a file containing the bookmarks should be created.");
		layout.setConstraints(bookmarks, constraints);
		panel.add(bookmarks);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		quiet = new JCheckBox("Quiet", options.quiet);
		quiet.addActionListener(new ActionListener() {
								   public void actionPerformed(ActionEvent e) {
									   options.quiet = ((JCheckBox)e.getSource()).isSelected();
								   }
							   });
		quiet.setMnemonic('q');
		quiet.setToolTipText("Minimal reporting during decoding process.");
		layout.setConstraints(quiet, constraints);
		panel.add(quiet);

		return(panel);
	}

	private JPanel getEOLPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("End-of-line type:");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		eolType = new JComboBox(new String [] {
								   "OS Default",
								   "Linefeed - UNIX",
								   "Carriage return - MAC",
								   "Carriage return/Linefeed - Windows/DOS",
							   });
		eolType.setSelectedIndex(options.eolType - 1);
		eolType.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 options.eolType = ((JComboBox)e.getSource()).getSelectedIndex();
									 }
								 });
		layout.setConstraints(eolType, constraints);
		panel.add(eolType);
		label.setLabelFor(eolType);
		label.setDisplayedMnemonic('e');
		label.setToolTipText("Used to indicate the End Of Line character to use.");

		return(panel);
	}

	private DecodeOptions options = null;
	private DecodeOptions resetOptions = null;

	private JTextField sourcePath = null;
	private JTextField destinationPath = null;
	private JCheckBox quiet = null;

	private JCheckBox bookmarks = null;
	private JComboBox eolType = null;

///////////////////////////////////////////////////////////////////////////////
// Implements the ProcessDoc Interface

	public void process() {
		options.sourcePath = sourcePath.getText();
		options.destinationFile = new File(destinationPath.getText());
		if(options.destinationFile.isDirectory()) {
			String titleText = options.sourcePath;
			int extention = titleText.lastIndexOf('.');
			if(extention > 0) {
				// has an extention (sure wish the File class has file-extention support)
				titleText = titleText.substring(0, extention) + ".txt";
			}
			options.destinationFile = new File(options.destinationFile, titleText);
		}
		makeDocGUI.docTask.process(options);
	}

	public HashMap getOptions() {
		HashMap opts = new HashMap();
		opts.put("sourcePath", sourcePath.getText());
		opts.put("destinationPath", destinationPath.getText());
		opts.put("quiet", "" + options.quiet);

		opts.put("bookmarks", "" + options.bookmarks);
		opts.put("eolType", "" + options.eolType);
		return(opts);
	}

	public void setOptions(HashMap options) {
		String value = null;
		value = (String)options.get("sourcePath");
		if(value != null) {
			sourcePath.setText(value);
		}
		value = (String)options.get("destinationPath");
		if(value != null) {
			destinationPath.setText(value);
		}
		value = (String)options.get("quiet");
		if(value != null) {
			this.options.quiet = Boolean.valueOf(value).booleanValue();
			quiet.setSelected(this.options.quiet);
		}
		value = (String)options.get("bookmarks");
		if(value != null) {
			this.options.bookmarks = Boolean.valueOf(value).booleanValue();
			bookmarks.setSelected(this.options.bookmarks);
		}
		value = (String)options.get("eolType");
		if(value != null) {
			eolType.setSelectedIndex(Integer.parseInt(value) - 1);
		}
		resetOptions = (DecodeOptions)this.options.clone();
	}

	public String getID() {
		return("Decode");
	}

	public void reset() {
		if(resetOptions == null) {
			options = new DecodeOptions();
		}
		else {
			options = (DecodeOptions)resetOptions.clone();
		}

		if(options.sourcePath == null) {
			sourcePath.setText("");
		}
		else {
			sourcePath.setText(options.sourcePath);
		}
		if(options.destinationFile == null) {
			destinationPath.setText("");
		}
		else {
			destinationPath.setText(options.destinationFile.getAbsolutePath());
		}
		quiet.setSelected(options.quiet);
		bookmarks.setSelected(options.bookmarks);
		eolType.setSelectedIndex(options.eolType - 1);
	}

	public void setConfiguration(String configuration) {
		this.configuration = configuration;
	}

	public String getConfiguration() {
		return(configuration);
	}

	String configuration = null;

// implements ProcessDoc Interface
///////////////////////////////////////////////////////////////////////////////
}

