/*
 * EncodePanel.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: EncodePanel.java,v 1.4 2006/02/08 05:03:06 brisk Exp $
 */

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

// NOTE: for JDK 1.1.x get special collections
// from Sun http://java.sun.com/products/javabeans/infobus/collectionsreadme.html
// import com.sun.java.util.collections.*;
// NOTE: for JDK 1.2+
// import java.util.*;
import @COLLECTIONS@; // used by ant

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * <code>EncodePanel</code> is the Panel with all the encoding options.
 * @author Jeffrey A. Krzysztow
 * @version 1.4.0
 */
public final class EncodePanel extends JPanel implements ProcessDOC {

	public EncodePanel(MakeDocGUI makeDocGUI) {
		super(true);

		this.makeDocGUI = makeDocGUI;

		options = new EncodeOptions();

		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = null;
		JPanel panel = null;

		layout = new GridBagLayout();
		constraints = new GridBagConstraints();
		constraints.insets = new Insets(4, 4, 4, 4);

		setLayout(layout);

		constraints.anchor = GridBagConstraints.WEST;
		constraints.gridwidth = GridBagConstraints.REMAINDER;

		panel = getSourceDestinationPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		panel = getDatabaseInfoPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		panel = getProcessingOptionsPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		panel = getBookmarkPanel();
		layout.setConstraints(panel, constraints);
		add(panel);

		@JAVA2_START@
		new DropTarget(sourcePath, new DropTargetListener() {
						   public void drop(DropTargetDropEvent dtde) {
							   try {
								   Transferable tr = dtde.getTransferable();
								   if(dtde.isDataFlavorSupported(DataFlavor.stringFlavor)) {
									   dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
									   String filename = (String)tr.getTransferData(DataFlavor.stringFlavor);
									   sourcePath.setText(filename);
									   dtde.dropComplete(true);
								   }
								   else if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
									   dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
									   java.util.List files = (java.util.List)tr.getTransferData(DataFlavor.javaFileListFlavor);
									   for(Iterator iterator = (Iterator)files.iterator(); iterator.hasNext();) {
										   File file = (File)iterator.next();
										   sourcePath.setText(file.getAbsolutePath());
										   setTitle(file);
										   break; // only process first file in the list, for now
									   }
									   dtde.dropComplete(true);
								   }
								   else {
									   dtde.rejectDrop();
								   }
							   }
							   catch(IOException ioe) {
								   ioe.printStackTrace();
							   }
							   catch(UnsupportedFlavorException ufe) {
								   ufe.printStackTrace();
							   }
						   }
						   public void dragExit(DropTargetEvent dte) {
						   }
						   public void dragEnter(DropTargetDragEvent dtde) {
						   }
						   public void dragOver(DropTargetDragEvent dtde) {
						   }
						   public void dropActionChanged(DropTargetDragEvent dtde) {
						   }
					   });
		@JAVA2_END@
	}

	private MakeDocGUI makeDocGUI = null;

	private JPanel getSourceDestinationPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Source: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.weightx = 1;
		sourcePath = new JTextField(30);
		layout.setConstraints(sourcePath, constraints);
		panel.add(sourcePath);
		label.setLabelFor(sourcePath);
		label.setDisplayedMnemonic('s');
		label.setToolTipText("Specifies the source text input to convert to a PilotDOC.");

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0;
		button = new JButton("...");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 makeDocGUI.browse(sourcePath, "Source file", makeDocGUI.textFilter);
										 setTitle(new File(sourcePath.getText()));
									 }
								 });
		layout.setConstraints(button, constraints);
		panel.add(button);

		constraints.gridwidth = 1;
		label = new JLabel("Destination: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.weightx = 1;
		destinationFile = new JTextField(30);
		layout.setConstraints(destinationFile, constraints);
		panel.add(destinationFile);
		label.setLabelFor(destinationFile);
		label.setDisplayedMnemonic('d');
		label.setToolTipText("Specifies the PilotDOC containing the converted text.");

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0;
		button = new JButton("...");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 makeDocGUI.browse(destinationFile, "Destination file", makeDocGUI.pilotDocFilter);
									 }
								 });
		layout.setConstraints(button, constraints);
		panel.add(button);

		return(panel);
	}

	private JPanel getDatabaseInfoPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JPanel subPanel = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		subPanel = getTitlePanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		constraints.gridwidth = 1;
		compress = new JCheckBox("Compress", options.compress);
		compress.addActionListener(new ActionListener() {
									   public void actionPerformed(ActionEvent e) {
										   options.compress = ((JCheckBox)e.getSource()).isSelected();
									   }
								   });
		compress.setMnemonic('c');
		compress.setToolTipText("Compress the PilotDOC file.");
		layout.setConstraints(compress, constraints);
		panel.add(compress);

		makePrivate = new JCheckBox("Private", options.makePrivate);
		makePrivate.addActionListener(new ActionListener() {
										  public void actionPerformed(ActionEvent e) {
											  options.makePrivate = ((JCheckBox)e.getSource()).isSelected();
										  }
									  });
		makePrivate.setMnemonic('v');
		makePrivate.setToolTipText("Sets the PilotDOC to private.");
		layout.setConstraints(makePrivate, constraints);
		panel.add(makePrivate);

		subPanel = getCategoryPanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		subPanel = getDocumentTypePanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		return(panel);
	}

	private JPanel getTitlePanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Title: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		title = new JTextField(32);
		layout.setConstraints(title, constraints);
		panel.add(title);
		label.setLabelFor(title);
		label.setDisplayedMnemonic('t');
		label.setToolTipText("The title of the PilotDOC.");

		return(panel);
	}

	private JPanel getCategoryPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Category:");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		category = new JComboBox();
		for(int x = 0; x < 16; x++) {
			category.addItem("" + x);
		}
		category.setSelectedIndex(options.category);
		category.addActionListener(new ActionListener() {
									   public void actionPerformed(ActionEvent e) {
										   options.category = (byte)((JComboBox)e.getSource()).getSelectedIndex();
									   }
								   });
		layout.setConstraints(category, constraints);
		panel.add(category);
		label.setLabelFor(category);
		label.setDisplayedMnemonic('g');
		label.setToolTipText("Sets the category for the PilotDOC.");

		return(panel);
	}

	private JPanel getDocumentTypePanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Document ID:");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		docID = new JComboBox(DatabaseHeader.knownIDtext);
		docID.setSelectedItem(DatabaseHeader.convertIntToStringCreatorID(options.docID));
		docID.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										options.docID = DatabaseHeader.convertStringToTypeID((String)((JComboBox)e.getSource()).getSelectedItem());
									}
								});
		layout.setConstraints(docID, constraints);
		panel.add(docID);
		label.setLabelFor(docID);
		label.setDisplayedMnemonic('i');
		label.setToolTipText("Used to set the PilotDOC to a particular venders.");

		return(panel);
	}

	private JPanel getProcessingOptionsPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JPanel subPanel = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		subPanel = getUnwrapPanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		compressEOL = new JCheckBox("Compress EOL", options.compressEOL);
		compressEOL.addActionListener(new ActionListener() {
										  public void actionPerformed(ActionEvent e) {
											  options.compressEOL = ((JCheckBox)e.getSource()).isSelected();
										  }
									  });
		compressEOL.setMnemonic('m');
		compressEOL.setToolTipText("Compress multi-linefeeds out of the PilotDOC.");
		layout.setConstraints(compressEOL, constraints);
		panel.add(compressEOL);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		quiet = new JCheckBox("Quiet", options.quiet);
		quiet.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										options.quiet = ((JCheckBox)e.getSource()).isSelected();
									}
								});
		quiet.setMnemonic('q');
		quiet.setToolTipText("Minimal reporting during encoding process.");
		layout.setConstraints(quiet, constraints);
		panel.add(quiet);

		return(panel);
	}

	private JPanel getUnwrapPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		unwrap = new JCheckBox("Unwrap", options.unwrap);
		unwrap.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 options.unwrap = ((JCheckBox)e.getSource()).isSelected();
									 }
								 });
		unwrap.setMnemonic('u');
		unwrap.setToolTipText("Unwraps the text by paragraphs (i.e. single linefeeds are removed).");
		layout.setConstraints(unwrap, constraints);
		panel.add(unwrap);

		label = new JLabel("Minimum Line Length:");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		lineLength = new JTextField(4);
		lineLength.setText("" + options.lineLength);
		layout.setConstraints(lineLength, constraints);
		panel.add(lineLength);
		label.setLabelFor(lineLength);
		label.setDisplayedMnemonic('n');
		label.setToolTipText("The minimum length of a line to unwrap.");

		return(panel);
	}

	private JPanel getBookmarkPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JPanel subPanel = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		subPanel = getBookmarkFilePanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		constraints.gridwidth = 1;
		autoBookMark = new JCheckBox("Auto-bookmark", options.autoBookMark);
		autoBookMark.addActionListener(new ActionListener() {
										   public void actionPerformed(ActionEvent e) {
											   options.autoBookMark = ((JCheckBox)e.getSource()).isSelected();
										   }
									   });
		autoBookMark.setMnemonic('a');
		autoBookMark.setToolTipText("Auto-bookmark (See ReadMe.html for more details).");
		layout.setConstraints(autoBookMark, constraints);
		panel.add(autoBookMark);

		trimBmWhitespace = new JCheckBox("Trim bookmark whitespace", options.trimBmWhitespace);
		trimBmWhitespace.addActionListener(new ActionListener() {
											   public void actionPerformed(ActionEvent e) {
												   options.trimBmWhitespace = ((JCheckBox)e.getSource()).isSelected();
											   }
										   });
		trimBmWhitespace.setMnemonic('k');
		trimBmWhitespace.setToolTipText("Trim extra whitespace from beginning/end of the bookmark names.");
		layout.setConstraints(trimBmWhitespace, constraints);
		panel.add(trimBmWhitespace);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		subPanel = getBookmarkSortPanel();
		layout.setConstraints(subPanel, constraints);
		panel.add(subPanel);

		return(panel);
	}

	ExampleFileFilter bookmarkFilter = new ExampleFileFilter(new String [] {
																 "bm",
																 "txt",
																 "doc",
																 "bookmark",
																 "bookmarks",
															 },
															 "Bookmark Text Files"
															);

	private JPanel getBookmarkFilePanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Bookmarks: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.weightx = 1;
		bookmarkPath = new JTextField(30);
		bookmarkPath.setEditable(true);
		layout.setConstraints(bookmarkPath, constraints);
		panel.add(bookmarkPath);
		label.setLabelFor(bookmarkPath);
		label.setDisplayedMnemonic('b');
		label.setToolTipText("The file containing the bookmarks that is used to search the source file.");

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0;
		button = new JButton("...");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 makeDocGUI.browse(bookmarkPath, "Bookmark file", bookmarkFilter);
									 }
								 });
		layout.setConstraints(button, constraints);
		panel.add(button);

		return(panel);
	}

	private JPanel getBookmarkSortPanel() {
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JPanel panel = null;
		JLabel label = null;

		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets = new Insets(1, 1, 1, 1);

		label = new JLabel("Bookmark Sort:");
		layout.setConstraints(label, constraints);
		panel.add(label);

		constraints.gridwidth = GridBagConstraints.REMAINDER;
		sortBookMarks = new JComboBox(new String [] {
										  "None",
										  "Location",
										  "Alphabetically",
									  });
		sortBookMarks.setSelectedIndex(options.sortBookMarks);
		sortBookMarks.addActionListener(new ActionListener() {
											public void actionPerformed(ActionEvent e) {
												options.sortBookMarks = ((JComboBox)e.getSource()).getSelectedIndex();
											}
										});
		layout.setConstraints(sortBookMarks, constraints);
		panel.add(sortBookMarks);
		label.setLabelFor(sortBookMarks);
		label.setDisplayedMnemonic('o');
		label.setToolTipText("Used to set how/if the bookmarks are sorted after they have been found.");

		return(panel);
	}

	private EncodeOptions options = null;
	private EncodeOptions resetOptions = null;

	private JTextField sourcePath = null;
	private JTextField destinationFile = null;
	private JCheckBox quiet = null;

	private JTextField bookmarkPath = null;
	private JTextField title = null;
	private JCheckBox compress = null;
	private JComboBox docID = null;
	private JCheckBox compressEOL = null;
	private JCheckBox unwrap = null;
	private JTextField lineLength = null;
	private JCheckBox makePrivate = null;
	private JComboBox category = null;
	private JComboBox sortBookMarks = null;
	private JCheckBox autoBookMark = null;
	private JCheckBox trimBmWhitespace = null;

	private void setTitle(File file) {
		String titleText = title.getText();
		if(titleText.length() == 0) {
			// there is no title already set
			titleText = file.getName();
			int extention = titleText.lastIndexOf('.');
			if(extention > 0) {
				// has an extention (sure wish the File class has file-extention support)
				titleText = titleText.substring(0, extention);
				title.setText(titleText);
			}
		}
	}

///////////////////////////////////////////////////////////////////////////////
// Implements the ProcessDoc Interface

	public void process() {
		String text = null;
		options.sourcePath = sourcePath.getText();
		options.destinationFile = new File(destinationFile.getText());
		if(options.destinationFile.isDirectory()) {
			options.destinationFile = new File(options.destinationFile, options.title + ".pdb");
		}
		text = bookmarkPath.getText();
		if(text != null && text.equals("") == false) {
			options.bookMarkPath = new File(text);
		}
		options.title = title.getText();
		makeDocGUI.docTask.process(options);
	}

	public HashMap getOptions() {
		HashMap opts = new HashMap();
		opts.put("sourcePath", sourcePath.getText());
		opts.put("destinationFile", destinationFile.getText());
		opts.put("quiet", "" + options.quiet);

		opts.put("bookmarkPath", bookmarkPath.getText());
		opts.put("title", title.getText());
		opts.put("compress", "" + options.compress);
		opts.put("docID", "" + options.docID);
		opts.put("compressEOL", "" + options.compressEOL);
		opts.put("unwrap", "" + options.unwrap);
		opts.put("lineLength", "" + options.lineLength);
		opts.put("makePrivate", "" + options.makePrivate);
		opts.put("category", "" + options.category);
		opts.put("sortBookMarks", "" + options.sortBookMarks);
		opts.put("autoBookMark", "" + options.autoBookMark);
		opts.put("trimBmWhitespace", "" + options.trimBmWhitespace);
		return(opts);
	}

	public void setOptions(HashMap options) {
		String value = null;
		value = (String)options.get("sourcePath");
		if(value != null) {
			sourcePath.setText(value);
		}
		value = (String)options.get("destinationFile");
		if(value != null) {
			destinationFile.setText(value);
		}
		value = (String)options.get("quiet");
		if(value != null) {
			this.options.quiet = Boolean.valueOf(value).booleanValue();
			quiet.setSelected(this.options.quiet);
		}
		value = (String)options.get("bookmarkPath");
		if(value != null) {
			bookmarkPath.setText(value);
		}
		value = (String)options.get("title");
		if(value != null) {
			title.setText(value);
		}
		value = (String)options.get("compress");
		if(value != null) {
			this.options.compress = Boolean.valueOf(value).booleanValue();
			compress.setSelected(this.options.compress);
		}
		value = (String)options.get("docID");
		if(value != null) {
			docID.setSelectedItem(DatabaseHeader.convertIntToStringCreatorID(Integer.parseInt(value)));
		}
		value = (String)options.get("compressEOL");
		if(value != null) {
			this.options.compressEOL = Boolean.valueOf(value).booleanValue();
			compressEOL.setSelected(this.options.compressEOL);
		}
		value = (String)options.get("unwrap");
		if(value != null) {
			this.options.unwrap = Boolean.valueOf(value).booleanValue();
			unwrap.setSelected(this.options.unwrap);
		}
		value = (String)options.get("lineLength");
		if(value != null) {
			lineLength.setText(value);
		}
		value = (String)options.get("makePrivate");
		if(value != null) {
			this.options.makePrivate = Boolean.valueOf(value).booleanValue();
			makePrivate.setSelected(this.options.makePrivate);
		}
		value = (String)options.get("category");
		if(value != null) {
			category.setSelectedIndex(Integer.parseInt(value));
		}
		value = (String)options.get("sortBookMarks");
		if(value != null) {
			sortBookMarks.setSelectedIndex(Integer.parseInt(value));
		}
		value = (String)options.get("autoBookMark");
		if(value != null) {
			this.options.autoBookMark = Boolean.valueOf(value).booleanValue();
			autoBookMark.setSelected(this.options.autoBookMark);
		}
		value = (String)options.get("trimBmWhitespace");
		if(value != null) {
			this.options.trimBmWhitespace = Boolean.valueOf(value).booleanValue();
			trimBmWhitespace.setSelected(this.options.trimBmWhitespace);
		}
		resetOptions = (EncodeOptions)this.options.clone();
	}

	public String getID() {
		return("Encode");
	}

	public void reset() {
		if(resetOptions == null) {
			options = new EncodeOptions();
		}
		else {
			options = (EncodeOptions)resetOptions.clone();
		}

		if(options.sourcePath == null) {
			sourcePath.setText("");
		}
		else {
			sourcePath.setText(options.sourcePath);
		}
		if(options.destinationFile == null) {
			destinationFile.setText("");
		}
		else {
			destinationFile.setText(options.destinationFile.getAbsolutePath());
		}
		quiet.setSelected(options.quiet);

		if(options.bookMarkPath == null) {
			bookmarkPath.setText("");
		}
		else {
			bookmarkPath.setText(options.bookMarkPath.getAbsolutePath());
		}
		if(options.title == null) {
			title.setText("");
		}
		else {
			title.setText(options.title);
		}
		compress.setSelected(options.compress);
		docID.setSelectedItem(DatabaseHeader.convertIntToStringCreatorID(options.docID));
		compressEOL.setSelected(options.compressEOL);
		unwrap.setSelected(options.unwrap);
		lineLength.setText("" + options.lineLength);
		makePrivate.setSelected(options.makePrivate);
		category.setSelectedIndex(options.category);
		sortBookMarks.setSelectedIndex(options.sortBookMarks);
		autoBookMark.setSelected(options.autoBookMark);
		trimBmWhitespace.setSelected(options.trimBmWhitespace);
	}

	public void setConfiguration(String configuration) {
		this.configuration = configuration;
	}

	public String getConfiguration() {
		return(configuration);
	}

	String configuration = null;

// implements ProcessDoc Interface
///////////////////////////////////////////////////////////////////////////////
}

