/*
 * MakeDocG.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MakeDocG.java,v 1.5 2004/06/17 01:21:01 brisk Exp $
 */

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import com.BRiSK.INIFile.*;

/**
 * <code>MakeDocG</code> is the main GUI application used to manipulate PilotDocs
 * @author Jeffrey A. Krzysztow
 * @version 1.2.1
 */
public final class MakeDocG extends JPanel implements TaskFinishedListener {
	
	final static String version = "1.2.1";

	public MakeDocG() {
		super(true);
		
		try {
			// Force SwingSet to come up in the Cross Platform L&F
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			// If you want the System L&F instead, comment out the above line and
			// uncomment the following:
			// UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception exc) {
			System.err.println("Error loading L&F: " + exc);
			System.exit(1);
		}
	
		setBorder(BorderFactory.createEtchedBorder());
		setLayout(new BorderLayout());

		JPanel panel = null;
		GridBagLayout layout = null;
		GridBagConstraints constraints = null;
		JLabel label = null;
		JButton button = null;

		// commands line panel
		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridheight = 1;
		constraints.gridwidth = 1;
		constraints.weightx = 0.0;
		label = new JLabel("Command: ");
		layout.setConstraints(label, constraints);
		panel.add(label);

		commands = new JComboBox();
		commands.setEditable(true);
		commands.setMaximumRowCount(10);
		commands.getEditor().addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (e != null) {
						String commandLine = (String)e.getActionCommand();
						if (commandLine != null) {
							mruComboBox(commands, commandLine, 10);
							runMakeDocJ(commandLine);
							// clear "commands line"
							commands.setSelectedIndex(-1);
						}
					}
				}
			}
		);
		commands.setSelectedItem(null);
		constraints.gridx++;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 1.0;
		layout.setConstraints(commands, constraints);
		panel.add(commands);
		add("North", panel);

		// output of MakeDocJ panel
		output = new JTextArea();
		output.setFont(new Font("monospaced", Font.PLAIN, 12));
		output.setRows(10);
		output.setColumns(80);
		output.setEditable(false);
		// output.setWrapStyleWord(true);
		// output.setLineWrap(true);
		JScrollPane scroller = new JScrollPane();
		JViewport viewport = scroller.getViewport();
		viewport.add(output);
		add("Center", scroller);

		docTask = new MakeDocGTask(output);
		docTask.addTaskFinishedListeners(this);
		
		// current directory panel
		panel = new JPanel();
		layout = new GridBagLayout();
		panel.setLayout(layout);
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridheight = 1;
		constraints.gridwidth = 1;
		constraints.weightx = 0.0;
		label = new JLabel("Directory: ");
		layout.setConstraints(label, constraints);
		panel.add(label);
		
		directories = new JComboBox();
		directories.setEditable(true);
		directories.setMaximumRowCount(10);
		directories.getEditor().addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (e != null) {
						String directory = (String)e.getActionCommand();
						if (directory != null) {
							mruComboBox(directories, directory, 10);
							setCurrentDirectory(directory);
						}
					}
				}
			}
		);
		directories.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e != null) {
						if (e.getStateChange() == e.SELECTED) {
							String directory = (String)e.getItem();
							if (directory != null && directory.equals("") == false) {
								mruComboBox(directories, directory, 10);
								setCurrentDirectory(directory);
							}
						}
					}
				}
			}
		);
		directories.setSelectedItem(null);
		constraints.gridx++;
		constraints.gridwidth = GridBagConstraints.RELATIVE;
		constraints.weightx = 1.0;
		layout.setConstraints(directories, constraints);
		panel.add(directories);

		button = new JButton("...");
		button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					browse();
				}
			}
		);
		constraints.gridx++;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 0.0;
		layout.setConstraints(button, constraints);
		panel.add(button);
		add("South", panel);
		
		browser = new JFileChooser();
		browser.setDialogType(JFileChooser.CUSTOM_DIALOG);
		browser.setDialogTitle("Select Directory...");
		browser.setApproveButtonText("Select");
		browser.setApproveButtonMnemonic('s');
		browser.setApproveButtonToolTipText("Select a directory");
		browser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	}

	/**
	 * Most Recently Used ComboxBox manipulation.
	 */
	private void mruComboBox(JComboBox comboBox, String add, int maxItems) {
		if (add != null && add.equals("") == false) {
			try {
				String comboItem = (String)comboBox.getSelectedItem();
				if (comboItem == null) {
					comboBox.insertItemAt(add, 0);
				}
				else {
					if (comboItem.equals(add)) {
						if (comboBox.getSelectedIndex() > 0) {
							// must be an item already in the list, move it to the top
							comboBox.removeItemAt(comboBox.getSelectedIndex());
							comboBox.insertItemAt(comboItem, 0);
						}
					}
					else {
						// must be a modified item, so add like new
						comboBox.insertItemAt(add, 0);
					}
				}
			}
			catch (ArrayIndexOutOfBoundsException e) {
				// we get this is nothing is in the JComboBox yet
				// and we ask for the selected item, Good Job Sun <frown>
				comboBox.insertItemAt(add, 0);
			}
			if (comboBox.getItemCount() > maxItems) {
				comboBox.removeItemAt(maxItems);
			}
		}
	}

	/**
	 * Presents the use with the Directory Selector
	 * @since 1.0
	 */
	private void browse() {
		String cwdItem = (String)directories.getSelectedItem();
		if (cwdItem != null) {
			browser.setCurrentDirectory(new File(cwdItem));
		}
		if (browser.showDialog(this, null) == JFileChooser.APPROVE_OPTION) {
			File dir = browser.getSelectedFile();
			String directory = dir.toString();
			directories.setSelectedItem(directory);
			setCurrentDirectory(directory);
		}
	}

	/**
	 * Gets the current working directory.
	 * @return The current working directory.
	 * @since 1.0
	 */
	public static String getCurrentDirectory() {
		return System.getProperty("user.dir");
	}

	/**
	 * This method will alter the current directory property in the system property list.
	 * @param dir The directory to change to.
	 * @since 1.0
	 */
	public static void setCurrentDirectory(String dir) {
		// Get the system wide properties
		Properties system = System.getProperties();

		// The name of the property we are dealing with
		String key = "user.dir";

		// If the property is not already present
		// add it
		if (system.getProperty(key) == null) {
			system.put(key, dir);
			System.setProperties( system );
		}
		// Otherwise remove it, and add it -
		// efectively updating it.
		else {
			system.remove(key);
			system.put(key, dir);
			System.setProperties(system);
		}
	}

	/**
	 * Redirects StdOut and StdErr, then passes the "commands line" to be parsed and acted on.
	 * @param commandLine The commands line for MakeDocJ
	 */
	private void runMakeDocJ(String commandLine) {
		if(processing == false) {
			processing = true;
			getParent().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			docTask.process(commandLine);
		}
	}

	public void taskFinished() {
		processing = false;
		getParent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	private boolean processing = false;

	protected JComboBox commands = null;
	protected JComboBox directories = null;

	private JFileChooser browser = null;
	
	private JTextArea output = null;

	private MakeDocGTask docTask = null;

	/**
	 * Start of the application
	 * @param args arguments to MakeDocG application
	 * @since 1.0
	 */
	public static void main(String args[]) {
		final JFrame frame = new JFrame();
		frame.setTitle("MakeDocG v" + version + " (MakeDocJ v" + MakeDocJ.version + ") - MakeDocJ Redirector");
		frame.setBackground(Color.lightGray);
		frame.getContentPane().setLayout(new BorderLayout());
		final MakeDocG makeDocG = new MakeDocG();
		frame.getContentPane().add("Center", makeDocG);
		frame.addWindowListener(
			new WindowAdapter () {
				public void windowClosing(WindowEvent e) {
					exit(frame, makeDocG);
				}
			}
		);
		frame.pack();
		try {
			Point location = new Point(ini.getIntValue("MakeDocG-State", "LocationX"), ini.getIntValue("MakeDocG-State", "LocationY"));
			Dimension outerSize = new Dimension(ini.getIntValue("MakeDocG-State", "Width"), ini.getIntValue("MakeDocG-State", "Height"));
			frame.setLocation(location);
			frame.setSize(outerSize);
		}
		catch(SomePartNotFoundException pnfe) {
			// Center frame to screen
			Point location = frame.getLocation();
			Dimension outerSize = frame.getToolkit().getScreenSize();
			Dimension size = frame.getSize();
			location.x = location.x + (outerSize.width - size.width) / 2;
			location.y = location.y + (outerSize.height - size.height) / 2;
			frame.setLocation(location);
		}

		int count = -1;
		String item = null;
		try {
			count = ini.getIntValue("Commands", "Count");
			for (int x = 0; x < count; x++) {
				item = ini.getStringValue("Commands", "" + (x + 1));
				// makeDocG.commands.addItem(item);
				makeDocG.commands.insertItemAt(item, x);
			}
			makeDocG.commands.setSelectedIndex(-1);
			
			count = ini.getIntValue("Directories", "Count");
			for (int x = 0; x < count; x++) {
				item = ini.getStringValue("Directories", "" + (x + 1));
				// makeDocG.directories.addItem(item);
				makeDocG.directories.insertItemAt(item, x);
			}
			makeDocG.directories.setSelectedIndex(-1);
		}
		catch(SomePartNotFoundException pnfe) {
		}
		frame.show();
	}

	final static INIFile ini = new INIFile(System.getProperty("user.dir") + "/MakeDocJ.ini");

	/**
	 * Handle saving state for MakeDocG.
	 */
	static void exit(JFrame frame, MakeDocG makeDocG) {
		int count = makeDocG.commands.getItemCount();
		ini.setValue("Commands", "Count", count);
		for (int x = count; x > 0; x--) {
			ini.setValue("Commands", "" + x, (String)makeDocG.commands.getItemAt(x - 1));
		}

		count = makeDocG.directories.getItemCount();
		ini.setValue("Directories", "Count", count);
		for (int x = count; x > 0; x--) {
			ini.setValue("Directories", "" + x, (String)makeDocG.directories.getItemAt(x - 1));
		}
		Point location = frame.getLocation();
		Dimension outerSize = frame.getSize();
		ini.setValue("MakeDocG-State", "LocationX", location.x);
		ini.setValue("MakeDocG-State", "LocationY", location.y);
		ini.setValue("MakeDocG-State", "Width", outerSize.width);
		ini.setValue("MakeDocG-State", "Height", outerSize.height);
		try {
			ini.write();
		}
		catch (IOException e) {
		}
		System.exit(0);
	}

}

