/*
 * MakeDocGTask.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MakeDocGTask.java,v 1.3 2003/04/09 15:54:02 brisk Exp $
 */

import java.io.*;

import java.util.Vector;

import javax.swing.JTextArea;

/**
 * <code>MakeDocGTask</code> manages running the Encode/Decode process in
 * the background, capturing and redirecting the output of the process into
 * a JTextArea, and finally manages the notification of TaskFinishedListener.
 * @author Jeffrey A. Krzysztow
 * @version 1.0
 */
public class MakeDocGTask {
	
	MakeDocGTask(JTextArea output) {
		this.output = output;
	}

	private JTextArea output = null;
	
	private ByteArrayOutputStream out = new ByteArrayOutputStream() {
		public void flush() throws IOException {
			output.append(new String(buf, 0, count));
            output.setCaretPosition(output.getDocument().getLength());
			reset();
		}
	};

	private ByteArrayOutputStream err = new ByteArrayOutputStream() {
		public void flush() throws IOException {
			output.append(new String(buf, 0, count));
            output.setCaretPosition(output.getDocument().getLength());
			reset();
		}
	};

    /**
     * Called to start the task.
     */
    void process(String _cmdline) {
		done = false;
		final String cmdline = _cmdline;	// not sure why, but will not compile without constructor having a final
											// variable within inner class.
        final SwingWorker worker = new SwingWorker() {
            public Object construct() {
                return new MakeDoc(cmdline);
            }
			public void finished() {
				done = true;
				notifyTaskFinishedListeners();
			}
        };
        worker.start();
    }

	public boolean done() {
		return done;
	}

	private boolean done = false;

	public void addTaskFinishedListeners(TaskFinishedListener listener) {
		taskFinishedListeners.addElement(listener);
	}

	public void removeTaskFinishedListeners(TaskFinishedListener listener) {
		taskFinishedListeners.removeElement(listener);
	}

	private void notifyTaskFinishedListeners() {
		for(int l = 0; l < taskFinishedListeners.size(); l++) {
			((TaskFinishedListener)taskFinishedListeners.elementAt(l)).taskFinished();
		}
	}
	
	private Vector taskFinishedListeners = new Vector();

	class MakeDoc {
		public MakeDoc(String cmdline) {
			// "clean up" last commands
			output.setText("");

			// redirect stdout
			PrintStream oStdout = System.out;
			out.reset();
			PrintStream stdout = new PrintStream(out, true);
			System.setOut(stdout);

			// redirect stderr
			PrintStream oStderr = System.err;
			err.reset();
			PrintStream stderr = new PrintStream(err, true);
			System.setErr(stderr);

			MakeDocJ.processLine(cmdline, false);

			// put back original stdout
			System.setOut(oStdout);
			stdout.flush();
			stdout.close();

			// put back original stderr
			System.setErr(oStderr);
			stderr.flush();
			stderr.close();
		}
	}
}
