/*
 * MakeDocGUI.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MakeDocGUI.java,v 1.9 2006/02/09 19:12:13 brisk Exp $
 */

import java.awt.*;
import java.awt.event.*;

import java.io.*;

// NOTE: for JDK 1.1.x get special collections
// from Sun http://java.sun.com/products/javabeans/infobus/collectionsreadme.html
// import com.sun.java.util.collections.*;
// NOTE: for JDK 1.2+
//import java.util.*;
import @COLLECTIONS@; // used by ant

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import com.BRiSK.INIFile.*;

/**
 * <code>MakeDocGUI</code> is the main GUI application used to manipulate PilotDocs
 * @author Jeffrey A. Krzysztow
 * @version 1.5.1
 */
public final class MakeDocGUI extends JPanel implements TaskFinishedListener {

	final static String version = "1.5.2";

	public MakeDocGUI() {
		super(true);

		try {
			boolean nativeLookAndFeel = false;
			try {
				nativeLookAndFeel = ini.getBooleanValue(lookAndFeel);
				if(nativeLookAndFeel) {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
			}
			catch(SomePartNotFoundException e) {
			}
			finally {
				if(nativeLookAndFeel == false) {
					UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
				}
			}
		}
		catch(Exception exc) {
			System.err.println("Error loading L&F: " + exc);
			System.exit(1);
		}

		setBorder(BorderFactory.createEtchedBorder());
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		tabbedPane = new JTabbedPane();
		EncodePanel encodePanel = new EncodePanel(this);
		tabbedPane.addTab("Encode", encodePanel);
		DecodePanel decodePanel = new DecodePanel(this);
		tabbedPane.addTab("Decode", decodePanel);
		add(tabbedPane);

		try {
			String tab = ini.getStringValue(stateTab);
			Component component = null;
			if(encodePanel.getID().equals(tab)) {
				component = encodePanel;
			}
			else if(decodePanel.getID().equals(tab)) {
				component = decodePanel;
			}
			tabbedPane.setSelectedComponent(component);
		}
		catch(SomePartNotFoundException e) {
		}

		add(getButtonPanel());

		output = new JTextArea(10, 80);
		output.setFont(new Font("monospaced", Font.PLAIN, 12));
		output.setEditable(false);
		JScrollPane scrollPane = new JScrollPane(output);
		add(scrollPane);

		MakeDocJ.fromGUI = true;

		docTask = new MakeDocGUITask(output);
		docTask.addTaskFinishedListeners(this);

	}

	private JTextArea output = null;

	JTabbedPane tabbedPane = null;

	static JFrame theApp = null;

	MakeDocGUITask docTask = null;

	public static void main(String args[]) {
		theApp = new JFrame();
		theApp.setTitle("MakeDocGUI v" + version + " (MakeDocJ v" + MakeDocJ.version + ")");
		theApp.setBackground(Color.lightGray);
		theApp.getContentPane().setLayout(new BorderLayout());
		final MakeDocGUI makeDocGUI = new MakeDocGUI();
		theApp.getContentPane().add("Center", makeDocGUI);
		theApp.addWindowListener(new WindowAdapter () {
									 public void windowClosing(WindowEvent w) {
										 exit(theApp, makeDocGUI);
									 }
								 });
		theApp.pack();
		theApp.setResizable(false);
		try {
			Point location = new Point(ini.getIntValue(stateLX), ini.getIntValue(stateLY));
			theApp.setLocation(location);
		}
		catch(SomePartNotFoundException e) {
			// Center frame to screen
			Point location = theApp.getLocation();
			Dimension outerSize = theApp.getToolkit().getScreenSize();
			Dimension size = theApp.getSize();
			location.x = location.x + (outerSize.width - size.width) / 2;
			location.y = location.y + (outerSize.height - size.height) / 2;
			theApp.setLocation(location);
		}
		theApp.show();
	}

	final static INIFile ini = new INIFile(System.getProperty("user.dir") + "/MakeDocJ.ini");
	final static SectionKeyPair stateLX = new SectionKeyPair("MakeDocGUI-State", "LocationX");
	final static SectionKeyPair stateLY = new SectionKeyPair("MakeDocGUI-State", "LocationY");
	final static SectionKeyPair stateTab = new SectionKeyPair("MakeDocGUI-State", "Tab");
	final static SectionKeyPair lookAndFeel = new SectionKeyPair("MakeDocGUI-State", "NativeLookAndFeel");


	/**
	 * Handle saving state for MakeDocGUI.
	 */
	static void exit(JFrame frame, MakeDocGUI makeDocGUI) {
		Point location = frame.getLocation();
		ini.setValue(stateLX, location.x);
		ini.setValue(stateLY, location.y);
		ini.setValue(stateTab, ((ProcessDOC)makeDocGUI.tabbedPane.getSelectedComponent()).getID());
		try {
			ini.write();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		System.exit(0);
	}

	ExampleFileFilter pilotDocFilter =
	new ExampleFileFilter(new String [] {
							  "prc",
							  "pdb",
						  },
						  "PilotDoc Files"
						 );

	ExampleFileFilter textFilter =
	new ExampleFileFilter(new String [] {
							  "txt",
							  "doc",
						  },
						  "Text Files"
						 );

	private JFileChooser browser = null;

	void browse(JTextField textField, String title, ExampleFileFilter fileFilter) {
		if(browser == null) {
			browser = new JFileChooser();
		}
		browser.setDialogType(JFileChooser.CUSTOM_DIALOG);
		browser.setApproveButtonMnemonic('s');
		browser.setDialogTitle(title);
		if(fileFilter != null) {
			browser.setFileFilter(fileFilter);
		}
		String file = textField.getText();
		if(file != null && file.equals("") == false) {
			browser.setSelectedFile(new File(file));
		}
		else {
			browser.setSelectedFile(null);
		}
		if(browser.showDialog(this, "Select") == JFileChooser.APPROVE_OPTION) {
			File selectedFile = browser.getSelectedFile();
			try {
				textField.setText(selectedFile.getCanonicalPath());
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
	}

	private JPanel getButtonPanel() {
		GridLayout layout = null;
		JPanel panel = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridLayout(1, 0);
		panel.setLayout(layout);

		processButton = new JButton("Process");
		processButton.addActionListener(new ActionListener() {
											public void actionPerformed(ActionEvent a) {
												Component tab = tabbedPane.getSelectedComponent();
												getParent().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
												processButton.setEnabled(false);
												((ProcessDOC)tab).process();
											}
										});
		processButton.setMnemonic('p');
		processButton.setToolTipText("Starts the encode/decode process");
		panel.add(processButton);

		button = new JButton("Reset");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent e) {
										 ((ProcessDOC)tabbedPane.getSelectedComponent()).reset();
									 }
								 });
		button.setMnemonic('r');
		button.setToolTipText("Resets current tab to default values");
		panel.add(button);

		configButton = new JButton("Configuration");
		configButton.addActionListener(new ActionListener() {
										   public void actionPerformed(ActionEvent a) {
											   Component tab = tabbedPane.getSelectedComponent();
											   ConfigurationDialog dialog = new ConfigurationDialog(theApp, configButton, ini, ((ProcessDOC)tab));
											   dialog.show();
										   }
									   });
		configButton.setMnemonic('f');
		configButton.setToolTipText("Save, Load, or Delete configurations");
		panel.add(configButton);

		return(panel);
	}

	public void taskFinished() {
		processButton.setEnabled(true);
		getParent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	private JButton configButton = null;
	private JButton processButton = null;
}

class ConfigurationDialog extends JDialog {

	ConfigurationDialog(Frame owner, Component component, INIFile ini, ProcessDOC process) {
		super(owner, true);

		this.ini = ini;
		this.process = process;
		this.id = process.getID();

		Container container = getContentPane();
		container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

		section = new JTextField();
		container.add(section);
		section.addActionListener(new ActionListener() {
									  public void actionPerformed(ActionEvent e) {
										  save();
									  }
								  });

		try {
			String [] keys = ini.getKeys(id);
			model = new DefaultListModel();
			list = new JList(model);
			for(int i = 0; i < keys.length; i++) {
				model.addElement(keys[i]);
			}
			list.setSelectedValue(process.getConfiguration(), true);
			list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
			list.addListSelectionListener(new ListSelectionListener() {
											  public void valueChanged(ListSelectionEvent e) {
												  if(e.getValueIsAdjusting()) {
													  return;
												  }
												  section.setText((String)(list.getSelectedValue()));
											  }
										  });
			list.addMouseListener(new MouseAdapter() {
									  public void mouseClicked(MouseEvent e) {
										  if(e.getClickCount() == 1) {
											  section.setText((String)(list.getSelectedValue()));
										  }
										  if(e.getClickCount() == 2) {
											  load.doClick();
										  }
									  }
								  });
			JScrollPane listScroller = new JScrollPane(list);
			container.add(listScroller);
		}
		catch(SectionNotFoundException e) {
			// this "cannot" happen as the list is generated by having already read the sections
			// from the INI file in the first place
		}

		container.add(getButtonPanel());
		pack();
		setLocationRelativeTo(component);
	}

	JPanel getButtonPanel() {
		GridLayout layout = null;
		JPanel panel = null;
		JButton button = null;

		panel = new JPanel();
		layout = new GridLayout(1, 0);
		panel.setLayout(layout);

		button = new JButton("Save");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent a) {
										 save();
										 setVisible(false);
									 }
								 });
		button.setMnemonic('s');
		button.setToolTipText("Saves the current set of options to the name specified");
		panel.add(button);

		load = new JButton("Load");
		load.addActionListener(new ActionListener() {
								   public void actionPerformed(ActionEvent a) {
									   load();
									   setVisible(false);
								   }
							   });
		load.setMnemonic('l');
		load.setToolTipText("Loads the specified set of options");
		panel.add(load);

		button = new JButton("Delete");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent a) {
										 delete();
									 }
								 });
		button.setMnemonic('d');
		button.setToolTipText("Deletes the specified set of options");
		panel.add(button);

		button = new JButton("Close");
		button.addActionListener(new ActionListener() {
									 public void actionPerformed(ActionEvent a) {
										 setVisible(false);
									 }
								 });
		button.setMnemonic('c');
		button.setToolTipText("Close dialog");
		panel.add(button);

		return panel;
	}

	private void save() {
		String configuration = section.getText();
		ini.setValue(id, configuration, "");
		String section = configuration + ":" + id;
		HashMap options = process.getOptions();
		Iterator opIt = options.keySet().iterator();
		try {
			ini.removeSection(section);
		}
		catch(SectionNotFoundException e) {
			// we don't care if it does not exist
		}
		while(opIt.hasNext()) {
			String optKey = (String) opIt.next();
			String optValue = (String) options.get(optKey);
			ini.setValue(section, optKey, optValue);
		}
		process.setConfiguration(configuration);
	}

	private void load() {
		try {
			String configuration = section.getText();
			String section = configuration + ":" + id;
			String [] keys = ini.getKeys(section);
			HashMap options = new HashMap();
			String keyValue;
			for(int x = 0; x < keys.length; x++) {
				try {
					keyValue = ini.getStringValue(section, keys[x]);
					options.put(keys[x], keyValue);
				}
				catch(ValueNotFoundException e) {
					// there are valid "empty" keys
				}
				catch(KeyNotFoundException e) {
					// this "cannot" happen as the list is generated by
					// having already read the keys from the INI file in
					// the first place
				}
			}
			process.setOptions(options);
			process.setConfiguration(configuration);
		}
		catch(SectionNotFoundException e) {
			e.printStackTrace();
		}
	}

	private void delete() {
		String configuration = section.getText();
		String section = configuration + ":" + id;
		try {
			ini.removeSection(section);
			ini.removeKey(id, configuration);
			model.removeElement(configuration);
		}
		catch(SomePartNotFoundException e) {
			e.printStackTrace();
		}
	}

	JTextField section = null;
	JButton load = null;
	JList list = null;
	DefaultListModel model = null;
	INIFile ini = null;
	String id = null;
	ProcessDOC process = null;
}

