/*
 * ReadDocJ.java
 *
 * Copyright 2003 by BRiSK Software,
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ReadDocJ.java,v 1.3 2003/08/02 22:28:41 brisk Exp $
 */

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.io.*;

// NOTE: for JDK 1.1.x get special collections
// from Sun http://java.sun.com/products/javabeans/infobus/collectionsreadme.html
// import com.sun.java.util.collections.*;
// NOTE: for JDK 1.2+
//import java.util.*;
import @COLLECTIONS@; // used by ant

import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import com.BRiSK.INIFile.*;

import gnu.regexp.*;

/**
 * <code>ReadDocJ</code> is the main application to read PilotDoc files
 * @author Jeffrey A. Krzysztow
 * @version 2.0
 */
public class ReadDocJ extends JFrame {
	private final static String version = "2.0.0";

	/**
	 * <code>SearchDlg</code> class maintains the search (find) dialog
	 * @author Jeffrey A. Krzysztow
	 * @since 1.0
	 */
	class SearchDlg extends JDialog implements ItemListener, KeyListener {
		JTextField searchFor = new JTextField(30);
		JCheckBox regex = new JCheckBox("Regular expression");
		JCheckBox nocase = new JCheckBox("Case insensitive");
		RESyntax reSyntax = null;

		/**
		 * Constructs the search dialog
		 * @param frame the parent frame
		 * @since 1.0
		 */
		SearchDlg(JFrame frame) {
			super(frame, false);

			setTitle("Search");

			GridBagLayout gbl = new GridBagLayout();
			GridBagConstraints gbc = new GridBagConstraints();
			Container contentPane = getContentPane();

			contentPane.setLayout(gbl);

			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dispose();
				}
			});

			setResizable(false);

			JToolBar toolBar = new JToolBar();
			toolBar.setFloatable(false);
			JButton button = null;

			button = new JButton("Find");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					find();
				}
			});
			button.addKeyListener(this);
			button.setMnemonic('f');
			toolBar.add(button);

			getRootPane().setDefaultButton(button);

			regex.addItemListener(this);
			regex.setMnemonic('r');
			toolBar.add(regex);

			nocase.addItemListener(this);
			nocase.setEnabled(false);
			nocase.setMnemonic('n');
			toolBar.add(nocase);

			searchFor.addKeyListener(this);
			gbc.gridx = GridBagConstraints.REMAINDER;
			gbl.setConstraints(searchFor, gbc);
			contentPane.add(searchFor);

			gbc.gridx = GridBagConstraints.REMAINDER;
			gbl.setConstraints(toolBar, gbc);
			contentPane.add(toolBar);

			pack();

			// Center dialog to parent frame
			Point location = frame.getLocation();
			Dimension outerSize = frame.getSize();
			Dimension size = getSize();
			location.x = location.x + (outerSize.width - size.width) / 2;
			location.y = location.y + (outerSize.height - size.height) / 2;
			setLocation(location);

			searchFor.requestFocus();

			reSyntax = new RESyntax(RESyntax.RE_SYNTAX_PERL5);
			reSyntax.setLineSeparator("\n");
		}

		/**
		 * handles to find button to search for text
		 * @since 1.0
		 */
		void find() {
			if(pilotDocLoaded == Boolean.TRUE) {
				String text = searchFor.getText();
				if(text != null) {
					if(regex.isSelected()) {
						try {
							RE searchRegex = new RE(text, RE.REG_MULTILINE | (nocase.isSelected() ? RE.REG_ICASE : 0), reSyntax);
							try {
								REMatch reMatch = searchRegex.getMatch(docText, doc.getCaretPosition() + 1);
								if(reMatch.getStartIndex() > -1) {
									doc.setCaretPosition(reMatch.getStartIndex());
									doc.select(reMatch.getStartIndex(), reMatch.getEndIndex() - reMatch.getStartIndex());
								}
							}
							catch(IllegalArgumentException e) {
							}
							finally {
							}
						}
						catch(REException e) {
						}
						finally {
						}
					}
					else {
						int pos = docText.indexOf(text);
						if(pos > 0) {
							doc.setCaretPosition(pos);
							doc.select(pos, pos + text.length());
						}
					}
				}
			}
		}

		/**
		 * implements ItemListener interface
		 * @param e ItemEvent
		 * @see java.event.ItemListener#itemStateChanged
		 * @since 1.0
		 */
		public void itemStateChanged(ItemEvent e) {
			if(e.getSource() == regex) {
				if(e.getStateChange() == ItemEvent.SELECTED) {
					nocase.setEnabled(true);
				}
				else if(e.getStateChange() == ItemEvent.DESELECTED) {
					nocase.setEnabled(false);
				}
			}
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyPressed
		 * @since 1.0
		 */
		public void keyPressed(KeyEvent e) {
			if(e.getKeyCode() == KeyEvent.VK_ENTER) {
				find();
			}
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyReleased
		 * @since 1.0
		 */
		public void keyReleased(KeyEvent e) {
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyTyped
		 * @since 1.0
		 */
		public void keyTyped(KeyEvent e) {
		}

	}

	/**
	 * <code>BookMarkDlg</code> class maintains the bookmark dialog
	 * @author Jeffrey A. Krzysztow
	 * @version 1.1
	 */
	class BookMarkDlg extends JDialog implements MouseListener, KeyListener {
		JList list = new JList();
		JTextField bookmarkName = new JTextField(16);

		/**
		 * Constructs the bookmark dialog
		 * @param frame the parent frame
		 * @since 1.0
		 */
		BookMarkDlg(JFrame frame) {
			super(frame, false);

			setTitle("Bookmarks");

			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					dispose();
				}
			});

			JPanel northPanel = new JPanel();
			GridBagLayout gbl = new GridBagLayout();
			GridBagConstraints gbc = new GridBagConstraints();

			Container contentPane = getContentPane();

			northPanel.setLayout(gbl);

			// bookmarkName.addKeyListener(this);
			gbc.gridx = GridBagConstraints.REMAINDER;
			gbl.setConstraints(bookmarkName, gbc);
			northPanel.add(bookmarkName);

			JToolBar toolBar = new JToolBar();
			toolBar.setFloatable(false);
			JButton button = null;

			button = new JButton("Go");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					go();
				}
			});
			button.setMnemonic('g');
			toolBar.add(button);
			getRootPane().setDefaultButton(button);

			button = new JButton("Add");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					add();
				}
			});
			button.setMnemonic('a');
			toolBar.add(button);

			button = new JButton("Delete");
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					delete();
				}
			});
			button.setMnemonic('d');
			toolBar.add(button);

			gbc.gridx = GridBagConstraints.REMAINDER;
			gbl.setConstraints(toolBar, gbc);
			northPanel.add(toolBar);

			contentPane.add(northPanel, BorderLayout.NORTH);

			list.setVisibleRowCount(10);
			list.addKeyListener(this);
			list.addMouseListener(this);
			JScrollPane scrollList = new JScrollPane(list);
			contentPane.add(scrollList, BorderLayout.CENTER);

			pack();

			// Center dialog to parent frame
			Point location = frame.getLocation();
			Dimension outerSize = frame.getSize();
			Dimension size = getSize();
			location.x = location.x + (outerSize.width - size.width) / 2;
			location.y = location.y + (outerSize.height - size.height) / 2;
			setLocation(location);
		}

		/**
		 * handles the go button to go to the bookmark specified
		 * @since 1.0
		 */
		void go() {
			if(pilotDocLoaded == Boolean.TRUE) {
				BookMarkHelperRD bm = null;
				String lookingFor = bookmarkName.getText();
				int t = list.getModel().getSize();
				for(int i = 0; i < t; i++) {
					bm = (BookMarkHelperRD)bookmarks.elementAt(i);
					if(lookingFor.equals(bm.name)) {
						try {
							Rectangle startLocation = doc.modelToView(doc.getLineStartOffset(doc.getLineOfOffset(bm.fileOffset)));
							scrollDoc.getViewport().setViewPosition(new Point(startLocation.x, startLocation.y));
						}
						catch(BadLocationException e) {
							System.err.println(e);
						}
						finally {
						}
						getParent().requestFocus();
						break;
					}
				}
			}
		}

		/**
		 * handles to add button to add the specified bookmark
		 * @since 1.0
		 */
		void add() {
			if(pilotDocLoaded == Boolean.TRUE) {
				String lookingFor = bookmarkName.getText();
				BookMarkHelperRD bm = null;
				int t = bookmarks.size();
				boolean addBookmark = true;
				for(int i = 0; i < t; i++) {
					bm = (BookMarkHelperRD)bookmarks.elementAt(i);
					if(lookingFor.equals(bm.name)) {
						addBookmark = false;
						break;
					}
				}
				if(addBookmark) {
					bm = new BookMarkHelperRD();
					bm.name = lookingFor;
					bm.fileOffset = doc.getCaretPosition();
					bookmarks.addElement(bm);
				}
				else {
					status.setText("Bookmark name already in use");
				}
			}
		}

		/**
		 * handles the delete button to delete the specified bookmark
		 * @since 1.0
		 */
		void delete() {
			if(pilotDocLoaded == Boolean.TRUE) {
				BookMarkHelperRD bm = null;
				String lookingFor = bookmarkName.getText();
				int t = bookmarks.size();
				for(int i = 0; i < t; i++) {
					bm = (BookMarkHelperRD)bookmarks.elementAt(i);
					if(lookingFor.equals(bm.name)) {
						bookmarks.removeElement(bm);
						bookmarkName.setText("");
						break;
					}
				}
			}
		}

		/**
		 * updates the list of bookmarks
		 * @since 1.0
		 */
		public void updateBookmarks() {
			list.setModel(bookmarks);
			bookmarkName.setText("");
		}

		/**
		 * implements MouseListener interface
		 * @param e MouseEvent
		 * @see java.event.MouseListerner#mouseClicked
		 * @since 1.0
		 */
		public void mouseClicked(MouseEvent e) {
			if(e.getClickCount() == 2) {
				bookmarkName.setText(((BookMarkHelperRD)list.getSelectedValue()).name);
				go();
			}
		}

		/**
		 * implements MouseListener interface
		 * @param e MouseEvent
		 * @see java.event.MouseListerner#mouseEntered
		 * @since 1.0
		 */
		public void mouseEntered(MouseEvent e) {
		}

		/**
		 * implements MouseListener interface
		 * @param e MouseEvent
		 * @see java.event.MouseListerner#mouseExited
		 * @since 1.0
		 */
		public void mouseExited(MouseEvent e) {
		}

		/**
		 * implements MouseListener interface
		 * @param e MouseEvent
		 * @see java.event.MouseListerner#mousePressed
		 * @since 1.0
		 */
		public void mousePressed(MouseEvent e) {
		}

		/**
		 * implements MouseListener interface
		 * @param e MouseEvent
		 * @see java.event.MouseListerner#mouseReleased
		 * @since 1.0
		 */
		public void mouseReleased(MouseEvent e) {
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyPressed
		 * @since 1.0
		 */
		public void keyPressed(KeyEvent e) {
			if(e.getKeyCode() == KeyEvent.VK_ENTER) {
				bookmarkName.setText(((BookMarkHelperRD)list.getSelectedValue()).name);
				go();
			}
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyReleased
		 * @since 1.0
		 */
		public void keyReleased(KeyEvent e) {
		}

		/**
		 * implements KeyListener interface
		 * @param e KeyEvent
		 * @see java.event.KeyListener#keyTyped
		 * @since 1.0
		 */
		public void keyTyped(KeyEvent e) {
		}

	}

	final static INIFile ini = new INIFile(System.getProperty("user.dir") + "/ReadDocJ.ini");
	final static SectionKeyPair lookAndFeel = new SectionKeyPair("ReadDocJ-State", "NativeLookAndFeel");
	final static SectionKeyPair fileSelector = new SectionKeyPair("ReadDocJ-State", "NativeFileSelector");
	final static SectionKeyPair stateLX = new SectionKeyPair("ReadDocJ-State", "LocationX");
	final static SectionKeyPair stateLY = new SectionKeyPair("ReadDocJ-State", "LocationY");
	final static SectionKeyPair stateW = new SectionKeyPair("ReadDocJ-State", "Width");
	final static SectionKeyPair stateH = new SectionKeyPair("ReadDocJ-State", "Height");
	final static SectionKeyPair lastPilotDoc = new SectionKeyPair("ReadDocJ-State", "LoadLastPilotDoc");

	JTextArea doc = new JTextArea(20, 40);
	JScrollPane scrollDoc = new JScrollPane(doc);
	JLabel status = new JLabel();

	String docText = null;
	DefaultListModel bookmarks = new DefaultListModel();
	File pilotDoc = new File(".");
	Boolean pilotDocLoaded = Boolean.FALSE;
	int pilotDocLocation = 0;
	DatabaseHeader dbHeader = new DatabaseHeader();
	DocumentHeader docHeader = new DocumentHeader();

	SearchDlg searchDlg = null;
	BookMarkDlg bookmarkDlg = null;
	JFrame mainFrame = null;

	boolean nativeLookAndFeel = false;
	boolean nativeFileSelector = false;
	boolean ok2loadLastPilotDoc = false;

	/**
	 * Constructs the main frame of the application
	 * @param s title
	 * @since 1.0
	 */
	ReadDocJ() {
		super("ReadDocJ v" + version + " - no document");

		try {
			try {
				nativeLookAndFeel = ini.getBooleanValue(lookAndFeel);
				if(nativeLookAndFeel) {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
			}
			catch(SomePartNotFoundException e) {
			}
			finally {
				if(nativeLookAndFeel == false) {
					UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
				}
			}
		}
		catch(Exception exc) {
			System.err.println("Error loading L&F: " + exc);
			System.exit(1);
		}

		try {
			nativeFileSelector = ini.getBooleanValue(fileSelector);
			ok2loadLastPilotDoc = ini.getBooleanValue(lastPilotDoc);
		}
		catch(SomePartNotFoundException e) {
		}

		increamentInstances();

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				shutdown();
			}
		});

		JMenuBar menuBar = new JMenuBar();
		JMenu menu = null;
		JMenuItem menuItem = null;
		JCheckBoxMenuItem checkBoxMenuItem = null;

		menu = new JMenu();
		menu.setMnemonic('F');
		menu.setText("File");

		menuItem = new JMenuItem();
		menuItem.setMnemonic('O');
		menuItem.setText("Open...");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				open();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem();
		menuItem.setMnemonic('I');
		menuItem.setText("Info...");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				info();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem();
		menuItem.setMnemonic('N');
		menuItem.setText("New window");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				newInstance();
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem();
		menuItem.setMnemonic('X');
		menuItem.setText("Exit");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				shutdown();
			}
		});
		menu.add(menuItem);
		menuBar.add(menu);

		menu = new JMenu();
		menu.setMnemonic('P');
		menu.setText("Options");

		checkBoxMenuItem = new JCheckBoxMenuItem();
		checkBoxMenuItem.setMnemonic('L');
		checkBoxMenuItem.setText("Native Look and Feel");
		checkBoxMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				nativeLookAndFeel = ((JCheckBoxMenuItem)e.getSource()).isSelected();
				ini.setValue(lookAndFeel, nativeLookAndFeel);
			}
		});
		checkBoxMenuItem.setSelected(nativeLookAndFeel);
		menu.add(checkBoxMenuItem);
		menuBar.add(menu);
		
		checkBoxMenuItem = new JCheckBoxMenuItem();
		checkBoxMenuItem.setMnemonic('S');
		checkBoxMenuItem.setText("Native File Selector");
		checkBoxMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				nativeFileSelector = ((JCheckBoxMenuItem)e.getSource()).isSelected();
				ini.setValue(fileSelector, nativeFileSelector);
			}
		});
		checkBoxMenuItem.setSelected(nativeFileSelector);
		menu.add(checkBoxMenuItem);
		menuBar.add(menu);
		
		checkBoxMenuItem = new JCheckBoxMenuItem();
		checkBoxMenuItem.setMnemonic('P');
		checkBoxMenuItem.setText("Load Last PilotDoc");
		checkBoxMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ok2loadLastPilotDoc = ((JCheckBoxMenuItem)e.getSource()).isSelected();
				ini.setValue(lastPilotDoc, ok2loadLastPilotDoc);
			}
		});
		checkBoxMenuItem.setSelected(ok2loadLastPilotDoc);
		menu.add(checkBoxMenuItem);
		menuBar.add(menu);
		
		setJMenuBar(menuBar);

		Container contentPane = getContentPane();

		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);

		JButton button = null;

		button = new JButton("Open...");
		button.setMnemonic('O');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				open();
			}
		});
		toolBar.add(button);

		button = new JButton("Load...");
		button.setMnemonic('L');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				load();
			}
		});
		toolBar.add(button);

		button = new JButton("Info...");
		button.setMnemonic('I');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				info();
			}
		}
		);
		toolBar.add(button);

		button = new JButton("Search...");
		button.setMnemonic('S');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				search();
			}
		}
		);
		toolBar.add(button);

		button = new JButton("Bookmarks...");
		button.setMnemonic('B');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				bookmarks();
			}
		}
		);
		toolBar.add(button);

		button = new JButton("Top");
		button.setMnemonic('T');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				top();
			}
		}
		);
		toolBar.add(button);

		button = new JButton("Bottom");
		button.setMnemonic('M');
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				bottom();
			}
		}
		);
		toolBar.add(button);
		contentPane.add(toolBar, BorderLayout.NORTH);

		doc.setEditable(false);
		doc.setWrapStyleWord(true);
		doc.setLineWrap(true);
		doc.setTabSize(1);
		contentPane.add(scrollDoc, BorderLayout.CENTER);

		status.setText("ReadDocJ v" + version);
		contentPane.add(status, BorderLayout.SOUTH);

@JAVA2_START@
		new DropTarget(doc, new DropTargetListener() {
			public void drop(DropTargetDropEvent dtde) {
				try {
					Transferable tr = dtde.getTransferable();
					if(dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
						dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
						java.util.List files = (java.util.List)tr.getTransferData(DataFlavor.javaFileListFlavor);
						for(Iterator iterator = (Iterator)files.iterator(); iterator.hasNext();) {
							loadPilotDoc((File)iterator.next());
							break; // only process first file in the list, for now
						}
						dtde.dropComplete(true);
					}
					else {
						dtde.rejectDrop();
					}
				}
				catch(IOException ioe) {
					ioe.printStackTrace();
				}
				catch(UnsupportedFlavorException ufe) {
					ufe.printStackTrace();
				}
				finally {
				}
			}
			public void dragExit(DropTargetEvent dte) {}
			public void dragEnter(DropTargetDragEvent dtde) {}
			public void dragOver(DropTargetDragEvent dtde) {}
			public void dropActionChanged(DropTargetDragEvent dtde) {}
		});
@JAVA2_END@

		pack();

		try {
			setLocation(new Point(ini.getIntValue(stateLX),ini.getIntValue(stateLY)));
			setSize(new Dimension(ini.getIntValue(stateW),ini.getIntValue(stateH)));
		}
		catch(SomePartNotFoundException e) {
			// Center frame to screen
			Point location = getLocation();
			Dimension outerSize = getToolkit().getScreenSize();
			Dimension size = getSize();
			location.x = location.x + (outerSize.width - size.width) / 2;
			location.y = location.y + (outerSize.height - size.height) / 2;
			setLocation(location);
		}

		try {
			int mru = 0;
			while(true) {
				mruList.add(ini.getStringValue("ReadDocJ-State", "MRU" + mru));
				mru++;
			}
		}
		catch(SomePartNotFoundException e) {
			// throw away, gets us out of above loop
		}

		searchDlg = new SearchDlg(this);

		bookmarkDlg = new BookMarkDlg(this);

		mainFrame = this;

		show();
	}

	/**
	 * application start
	 * @param args command line arguments
	 * @since 1.0
	 */
	public static void main(String[] args) {
		ReadDocJ rdj = new ReadDocJ();
		if(args.length > 0) {
			rdj.loadPilotDoc(new File(args[0]));
		}
		else {
			rdj.loadLastPilotDoc();
		}
	}

	/**
	 * shuts downs the application
	 * @since 1.0
	 */
	void shutdown() {
		savePilotDocState();
		decrementInstances();
		if(isLastInstance()) {
			try {
				Point location = getLocation();
				ini.setValue(stateLX, location.x);
				ini.setValue(stateLY, location.y);
				Dimension size = getSize();
				ini.setValue(stateW, size.width);
				ini.setValue(stateH, size.height);
				for(int mru = 0; mru < mruList.size(); mru++) {
					ini.setValue("ReadDocJ-State", "MRU" + mru, (String)mruList.get(mru));
				}
				ini.write();
			}
			catch(Exception e) {
				e.printStackTrace();
			}
			finally {
			}
			System.exit(0);
		}
	}

	FileDialog fd = null;
	JFileChooser fc = null;

	/**
	 * handles the open button to open a PilotDoc file
	 * @since 1.0
	 */
	void open() {
		if(nativeFileSelector) {
			if(fd == null) {
				fd = new FileDialog(mainFrame);
			}
			fd.show();
			String file = fd.getFile();
			if(file != null) {						  	
				loadPilotDoc(new File(fd.getDirectory(), file));
			}
		}
		else {
			if(fc == null) {
				fc = new JFileChooser(pilotDoc);
				ExampleFileFilter filter = new ExampleFileFilter(new String [] { "prc", "pdb"}, "PilotDoc Files");
				fc.setFileFilter(filter);
			}
			if(fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
				loadPilotDoc(fc.getSelectedFile());
			}
		}
	}

	/**
	 * opens the PilotDoc
	 * @param pilotDocName the name of the PilotDoc to open
	 * @since 2.0
	 */
	void open(String pilotDocName) {
		loadPilotDoc(new File(pilotDocName));
	}

	/**
	 * loads the last pilotdoc, ie refresh
	 * @since 2.0
	 */
	void loadLastPilotDoc() {
		try {
			if(ok2loadLastPilotDoc) {
				open(ini.getStringValue("ReadDocJ-State", "MRU0"));
			}
		}
		catch(SomePartNotFoundException e) {
		}
	}

	/**
	 * handles the search button to present the search dialog
	 * @since 1.0
	 */
	void search() {
		if(searchDlg != null) {
			searchDlg.show();
		}
	}

	/**
	 * handles the bookmark button to present the bookmark dialog
	 * @since 1.0
	 */
	void bookmarks() {
		if(bookmarkDlg != null) {
			bookmarkDlg.show();
		}
	}

	/**
	 * handles the top button to go to the top of the PilotDoc
	 * @since 1.1
	 */
	void top() {
		if(docText != null) {
			doc.setCaretPosition(0);
		}
	}

	/**
	 * handles the bottom button to go to the bottom of the PilotDoc
	 * @since 1.1
	 */
	void bottom() {
		if(docText != null) {
			doc.setCaretPosition(docText.length());
		}
	}

	/**
	 * display a dialog with the PiloDoc information
	 * @since 2.0
	 */
	void info() {
		JOptionPane.showMessageDialog(this, dbHeader.toString() + "\n" + docHeader.toString(), dbHeader.name, JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * handles the new button to create a new instance of ReadDocJ
	 * @since 2.0
	 */
	void newInstance() {
		new ReadDocJ();
	}

	static int instances = 0;
	static Object instanceLock = new Object();

	/**
	 * increment the instance count
	 * @since 2.0
	 */
	void increamentInstances() {
		synchronized(instanceLock) {
			instances++;
		}
	}

	/**
	 * decrements the instance count
	 * @since 2.0
	 */
	void decrementInstances() {
		synchronized(instanceLock) {
			instances--;
		}
	}

	/**
	 * determine if all instances are ended
	 * @return <code>true</code> if all instances are gone
	 * @since 2.0
	 */
	boolean isLastInstance() {
		boolean retvalue = false;
		synchronized(instanceLock) {
			retvalue = (instances <= 0);
		}
		return(retvalue);
	}

	/**
	 * updates the status bar in a Swing friendly maner
	 * @param message the message to put on the status
	 * @since 2.0
	 */
	void updateStatus(final String message) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				status.setText(message);
				Thread.yield();
			}
		});
	}

	/**
	 * displays an error in a Swing friendly maner
	 * @param error te error to display
	 * @since 2.0
	 */
	void displayError(final JFrame frame, final String error) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					JOptionPane.showMessageDialog(frame, error, "Error", JOptionPane.ERROR_MESSAGE);
				}
			});
		}
		catch(Exception e) {
		}
	}

	/**
	 * appends the text to the JTextArea
	 * @since 2.0
	 */
	void append(final String text) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				doc.append(text);
			}
		});
	}

	/**
	 * loads the pilot doc file.
	 * @param docFile the PilotDoc File
	 * @since 2.0
	 */
	void loadPilotDoc(File docFile) {
		savePilotDocState();
		status.setText(" ");
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		doc.setText("");
		pilotDoc = docFile;
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				return(new LoadDoc(mainFrame));
			}
			public void finished() {
				if(pilotDocLoaded == Boolean.TRUE) {
					doc.setText(docText);
					Thread.yield(); // let the UI catchup
					setTitle("ReadDocJ v" + version + " - " + dbHeader.name);
					Thread.yield(); // let the UI catchup
					bookmarkDlg.updateBookmarks();
					Thread.yield(); // let the UI catchup
					add2mru(pilotDoc.getAbsolutePath());
					Thread.yield(); // let the UI catchup
					doc.setCaretPosition(pilotDocLocation);
					try {
						Rectangle startLocation = doc.modelToView(doc.getLineStartOffset(doc.getLineOfOffset(doc.getCaretPosition())));
						scrollDoc.getViewport().setViewPosition(new Point(startLocation.x, startLocation.y));
					}
					catch(BadLocationException e) {
					}
					updateStatus(dbHeader.name + " successfully loaded. " + pilotDocLocation);
				}
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
		};
		worker.start();
	}

	/**
	 * Class that runs in a thread to load the Pilot Doc
	 * @since 2.0
	 * @version 1.0
	 */
	class LoadDoc {
		public LoadDoc(JFrame frame) {
			synchronized(pilotDocLoaded) {
				pilotDocLoaded = Boolean.FALSE;
			}
			RandomAccessFile fin = null;
			try {
				bookmarks.clear(); // no bookmarks yet
				fin = new RandomAccessFile(pilotDoc, "r");
				if(decode(fin, pilotDoc)) {
					loadPilotDocState(frame);
					if(bookmarks.size() == 0) {
						processEmbeddedBookmarks();
					}

					synchronized(pilotDocLoaded) {
						pilotDocLoaded = Boolean.TRUE;
					}
				}
			}
			catch(FileNotFoundException e) {
				e.printStackTrace();
				updateStatus("The file " + pilotDoc.getName() + " could not be found");
				displayError(frame, "The file " + pilotDoc.getName() + " could not be found\n" + e.getMessage());
			}
			catch(IOException e) {
				e.printStackTrace();
				updateStatus("Error accessing file " + pilotDoc.getName());
				displayError(frame, "Error accessing file " + pilotDoc.getName() + "\n" + e.getMessage());
			}
			catch(Throwable e) {
				e.printStackTrace();
				updateStatus("Unable to load " + pilotDoc.getName());
				displayError(frame, "Unable to load " + pilotDoc.getName() + "\nIt might not be a valid PilotDoc file!");
			}
			finally {
				try {
					if(fin != null) {
						fin.close();
					}
				}
				catch(IOException e) {
				}
				finally {
				}
			}
		}
	}

	/**
	 * decodes the PilotDoc file
	 * @param fin      RandomAccessFile to the PilotDoc file
	 * @param file     File to the PilotDocs file
	 * @param dbHeader returns the DatabaseHeader record, must be created by caller
	 * @return <code>true</code> if successful, <code>false</code>if failed
	 * @throws IOException to the caller can handle any I/O errors
	 * @since 1.0
	 */
	boolean decode(final RandomAccessFile fin, final File file) throws IOException {
		updateStatus("Processing database header");
		dbHeader.read(fin);

		if(dbHeader.typeID != DatabaseHeader.TEXt || DatabaseHeader.isKnownCreatorID(dbHeader.creatorID) == false) {
			updateStatus(file.getName() + " is unknown format");
			return(false);
		}

		RecordIndex[] ri = new RecordIndex[dbHeader.numRecords];
		for(int i = 0; i < dbHeader.numRecords; i++) {
			updateStatus("Processing record index " + i + " of " + dbHeader.numRecords);
			ri[i] = new RecordIndex();
			ri[i].read(fin);
		}

		short unknown = fin.readShort();

		docHeader = new DocumentHeader();
		updateStatus("Processing document header");
		fin.seek(ri[0].fileOffset);
		docHeader.read(fin);

		if(docHeader.version != DocumentHeader.UNCOMPRESSED && docHeader.version != DocumentHeader.COMPRESSED) {
			updateStatus(file.getName() + " unknown file compression type: " + docHeader.version);
			return(false);
		}

		boolean compressed = (docHeader.version == DocumentHeader.COMPRESSED);
		ByteArrayOutputStream decompressedPilotDoc = new ByteArrayOutputStream(docHeader.storyLen);
		PilotDocRecord textBuffer = null;
		int dwRecLen;
		for(int i = 1; i <= docHeader.numRecords; i++) {
			updateStatus("Decoding " + i + " of " + docHeader.numRecords);
			if(i == ri.length - 1) {
				// for the last, use the file len
				dwRecLen = (int) fin.length() - ri[i].fileOffset;
			}
			else {
				dwRecLen = ri[i + 1].fileOffset - ri[i].fileOffset;
			}
			if(textBuffer == null || dwRecLen != textBuffer.length()) {
				textBuffer = new PilotDocRecord(dwRecLen);
			}
			fin.seek(ri[i].fileOffset);
			fin.read(textBuffer.buf);
			if(compressed) {
				textBuffer.decompress();
			}
			decompressedPilotDoc.write(textBuffer.buf, 0, textBuffer.length());
		}
		docText = decompressedPilotDoc.toString();

		/////////////////////////////////////////////////////////////
		// Handle encoded bookmarks if any
		if(docHeader.numRecords + 1 < dbHeader.numRecords) {
			int bmn = 1;
			int bmt = dbHeader.numRecords - docHeader.numRecords - 1;
			for(int i = docHeader.numRecords + 1; i < dbHeader.numRecords; i++, bmn++) {
				updateStatus("Processing internal bookmark " + bmn + " of " + bmt);
				fin.seek(ri[i].fileOffset);
				BookMarkHelperRD bm = new BookMarkHelperRD();
				bm.read(fin);
				bookmarks.addElement(bm);
			}
		}
		return(true);
	}

	/**
	 * processes the auto-bookmarks if present
	 * @since 1.1
	 */
	void processEmbeddedBookmarks() {
		char curChar;
		int startPos = -1;
		int endPos = -1;
		for(int curCharPos = docText.length() - 1; curCharPos > -1; curCharPos--) {
			curChar = docText.charAt(curCharPos);
			if(curChar == '<') {	// start sequence
				startPos = curCharPos;
				break;	// we are done looking
			}
			else if(curChar == '>') {	// end sequence
				endPos = curCharPos;
			}
			else if(!Character.isWhitespace(curChar) && endPos == -1) {
				break;
			}
		}
		if(startPos > -1 && endPos > -1) {
			String lookFor = docText.substring(startPos + 1, endPos);
			int found = 0;
			while((found = docText.indexOf(lookFor, found)) > 0) {
				if(docText.charAt(found - 1) == '\n') {
					BookMarkHelperRD bm = new BookMarkHelperRD();
					bm.fileOffset = found;
					int curCharPos = found += lookFor.length();
					for(; curCharPos < docText.length(); curCharPos++) {
						if(docText.charAt(curCharPos) == '\n') {
							while(Character.isWhitespace(docText.charAt(found))) {
								found++;
							}
							bm.name = docText.substring(found, curCharPos);
							bookmarks.addElement(bm);
							break;
						}
					}
					found = curCharPos;
				}
				else {
					found++;
				}
			}
		}
	}

	final private int bookmarkID = 0x4252534b;	// BRSK, so we "know" it's a "valid" bookmark file
	final private String pilotDocSection = "PilotDoc:";

	/**
	 * generates the ini section name for the PilotDoc
	 * @return the ini section name for the PilotDoc
	 * @since 2.0
	 */
	String getIniSectionName() {
		return pilotDocSection + pilotDoc.getAbsolutePath();
	}

	/**
	 * saves the "state" of the PilotDoc
	 * State is: Bookmarks, Caret Position
	 * @since 1.0
	 */
	void savePilotDocState() {
		if(pilotDocLoaded == Boolean.TRUE) {
			String sectionName = getIniSectionName();
			
			ini.setValue(sectionName, "LastModified", pilotDoc.lastModified());

			Point point = scrollDoc.getViewport().getViewPosition();
			pilotDocLocation = doc.viewToModel(point);
			updateStatus("Saving location " + pilotDocLocation);
			ini.setValue(sectionName, "Location", pilotDocLocation);
			
			int t = bookmarks.size();
			ini.setValue(sectionName, "Bookmarks", t);
			BookMarkHelperRD bm = null;
			for(int i = 0; i < t; i++) {
				bm = (BookMarkHelperRD)bookmarks.elementAt(i);
				updateStatus("Processing external bookmark " + (i+1) + " of " + t);
				ini.setValue(sectionName, "Bookmark" + i, bm.toINI());
			}
		}
	}

	/**
	 * loads the "state" of the PilotDoc
	 * State is: Bookmarks, Caret Position
	 * @param frame the frame to use if there are errors
	 * @since 1.0
	 */
	void loadPilotDocState(JFrame frame) {
		// see if there is an old leftover external .bm file that I used to do
		File pilotDocBM = new File(pilotDoc.getParent(), pilotDoc.getName() + ".bm");
		if(pilotDocBM.exists()) {
			if(pilotDocBM.lastModified() >= pilotDoc.lastModified()) {
				updateStatus("Processing extenal bookmarks");
				RandomAccessFile bin = null;
				try {
					bin = new RandomAccessFile(pilotDocBM, "rw");
					if(bin.readInt() == bookmarkID) {
						bookmarks.clear();
						BookMarkHelperRD bm = null;
						int count = bin.readInt();
						for(int x = 0; x < count; x++) {
							updateStatus("Processing external bookmark " + x + " of " + count);
							bm = new BookMarkHelperRD();
							bm.read(bin);
							bookmarks.addElement(bm);
						}
						updateStatus("Finished processing external bookmarks");
					}
					else {
						displayError(frame, "Invalid external bookmarks file.");
					}
				}
				catch(IOException e) {
					displayError(frame, "Error accessing file " + pilotDocBM.getName());
				}
				finally {
					try {
						bin.close();
					}
					catch(IOException e) {
					}
					finally {
					}
				}
			}
			else {
				displayError(frame, "External bookmark file will not be loaded, out of date.");
			}
			pilotDocBM.delete();
		}
		else {
			// get bookmarks from the INI
			String sectionName = getIniSectionName();
			try {
				if(ini.getLongValue(sectionName, "LastModified") >= pilotDoc.lastModified()) {
					pilotDocLocation = ini.getIntValue(sectionName, "Location");
					int t =  ini.getIntValue(sectionName, "Bookmarks");
					if(t > 0) {
						bookmarks.clear();
						BookMarkHelperRD bm = null;
						for(int i = 0; i < t; i++) {
							updateStatus("Processing external bookmark " + (i+1) + " of " + t);
							bm = new BookMarkHelperRD(ini.getStringValue(sectionName, "Bookmark" + i));
							bookmarks.addElement(bm);
						}
					}
				}
				else {
					displayError(frame, "External bookmark file will not be loaded, out of date.");
				}
			}
			catch(SomePartNotFoundException e) {
				// throw away
			}
			finally {
				try {
					ini.removeSection(sectionName);
				}
				catch(SectionNotFoundException e) {
					// throw it away
				}
			}
		}
	}

	int mruListSize = 10;
	ArrayList mruList = new ArrayList(mruListSize);

	/**
	 * @since 2.0
	 */
	void add2mru(String pilotDocFile) {
		mruList.remove(pilotDocFile); // if it is there, remove it
		mruList.add(0, pilotDocFile); // add it to the begining
		try {
			mruList.remove(mruListSize);
		}
		catch(IndexOutOfBoundsException e) {
			// we don't care
		}
	}

	/**
	 * @since 2.0
	 */
	void load() {
		if(mruList.size() > 0) {
			String [] fileList = (String [])mruList.toArray(new String [0]);
			String file = (String)JOptionPane.showInputDialog(
								mainFrame,
								"Select PilotDoc to Load",
								"Load PilotDoc",
								JOptionPane.PLAIN_MESSAGE,
								null,
								fileList,
								null
							);
			if(file != null) {
				open(file);
			}
		}
	}
}

/**
 * <code>BookMarkHelperRD</code> class is used to add the necessary support for ReadDocJ
 * @author Jeffrey A. Krzysztow
 * @version 1.1
 */
class BookMarkHelperRD extends BookMark {

	/**
	 * "Default" constructor
	 * @since 1.1
	 */
	BookMarkHelperRD () {
		super();
	}

	/**
	 * Constructor to create a BookMark via an INI entry
	 * @param ini the INI entry to parse
	 * @since 1.1
	 */
	BookMarkHelperRD(String ini) {
		this.ini = ini;
		try {
			StringTokenizer tokenizer = new StringTokenizer(ini, ",");
			String token = null;
			token = tokenizer.nextToken();
			fileOffset = Integer.parseInt(token);
			name = tokenizer.nextToken();
		}
		catch(NumberFormatException e) {
			// this should not happen
			e.printStackTrace();
		}
		catch(NoSuchElementException e) {
			// this should not happen
			e.printStackTrace();
		}
	}

	/**
	 * returns the string representation "we" want
	 * used by JList for holding bookmarks
	 * @return the name of the bookmark
	 * @since 1.0
	 */
	public String toString() {
		return(name);
	}

	/**
	 * Creates an INI entry
	 * @returns a string represenation to store in the INI file
	 * @since 1.1
	 */
	public String toINI() {
		if(ini == null) {
			ini = new String(fileOffset + "," + name);
		}
		return(ini);
	}
	private String ini = null;
}

